`define ACCADR  32'hfffe3000
`define INITADR	32'h20000000
`define MEMADR  32'h00000000
`define FLOPTADR  32'hfffe3000
`define FLMAADR  32'hfffe3004
`define FLINADR  32'hfffe3008
`define FLOUTADR  32'hfffe300C

`define SLOPTADR  32'hfffe37fc
`define SLINADR  32'hfffe3000
`define SLOUTADR  32'hfffe3800
`define SLPOLLADR		32'hfffe3ffc



`define FLOPT1 	32'h00000002
`define FLMA1 		32'h00000002
`define FLIN1 		32'h80000000
`define FLOUT1 	32'h90000000

`define FLALL	 	128'h90000040800000400000000100000002

`define SLOPT  32'h00000004
`define SLIN1  32'hbabebabe
`define SLIN2  32'hcafecafe
`define SLIN3  32'hdadedade
`define SLIN4  32'hcafebabe
`define SLIN5  32'h00001005
`define SLIN6  32'h00001006
`define SLIN7  32'h00001007
`define SLIN8  32'h00001008

`define IRQ 	128'h00000020000000200000002000000020
module a25_core_dummy
(
 input          i_clk,

 input          i_rst,

 input          i_irq, // Interrupt request, active high
 input           i_firq, // Fast Interrupt request, active high

 input          i_system_rdy, // Amber is stalled when this is low

// Wishbone Master I/F
 output reg [31:0]  o_wb_adr,
 output reg[15:0]  o_wb_sel,
 output reg        o_wb_we,
 input [127:0]  i_wb_dat,
 output reg[127:0] o_wb_dat,
 output reg        o_wb_cyc,
 output reg        o_wb_stb,
 input          i_wb_ack,
 input          i_wb_err,

// interface for exclusive op
 output         o_ld_excl
);


reg [31:0] state;
	

//statelist	
	localparam 	START = 32'h0,
					TSEL = 32'h1,
					M1 = 32'h2,
					M2 = 32'h3,
					M3 = 32'h4,
					M4 = 32'h5,
					M5 = 32'h6,
					M6 = 32'h7,
					
					IRQSET = 32'h2000,
					S1 = 32'h1001,
					S2 = 32'h1002,
					S3 = 32'h1003,
					S4 = 32'h1004,
					S5 = 32'h1005,
					S6 = 32'h1006,
					S7 = 32'h1007,
					S8 = 32'h1008,
					S9 = 32'h1009,
					Sa = 32'h100a;
					
				

always @(posedge i_clk)
begin
	if(i_rst)
	begin
		state = START;
		o_wb_adr <= 'h0;
		o_wb_sel <= 'h0;
		o_wb_we <= 'h0;
		o_wb_cyc <= 'h0;
		o_wb_stb <= 'h0;
		o_wb_dat <= 'h0;
	end
	else
	begin
		case(state)
			START:
			begin
				o_wb_adr <= 32'hfffe2008;
				o_wb_sel <= 16'h0f00;
				o_wb_we  <= 1;
				o_wb_cyc  <= 1;
				o_wb_stb  <= 1;
				o_wb_dat <= `IRQ;
				state = IRQSET;
				
			end
			IRQSET:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `INITADR;
					o_wb_sel <= 16'h000f;
					o_wb_we  <= 0;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					state = TSEL;
				
				end
			end
			TSEL:
			begin
				if(i_wb_ack)
				begin
					if(i_wb_dat[1])
					begin
						o_wb_adr <= `FLMAADR;
						o_wb_sel <= 16'h00f0;
						o_wb_we  <= 1;
						o_wb_cyc  <= 1;
						o_wb_stb  <= 1;
						o_wb_dat <= {`FLMA1,`FLMA1,`FLMA1,`FLMA1};
						state = M1;
					end
					else if(i_wb_dat[3:0] == 4'h1)
					begin
						o_wb_adr <= `SLINADR + 'h0;
						o_wb_sel <= 'h000f;
						o_wb_we <= 'h1;
						o_wb_cyc <= 'h1;
						o_wb_stb <= 'h1;
						o_wb_dat <= {`SLIN1,`SLIN1,`SLIN1,`SLIN1};
						state = S1;
					end
					else
					begin
						o_wb_adr <= 'h0;
						o_wb_sel <= 'h0;
						o_wb_we <= 'h0;
						o_wb_cyc <= 'h0;
						o_wb_stb <= 'h0;
						state = START;
					end
				end
			end
			M1:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `FLINADR;
					o_wb_sel <= 16'h0f00;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`FLIN1,`FLIN1,`FLIN1,`FLIN1};
					state = M2;
				end
			end
			M2:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `FLOUTADR;
					o_wb_sel <= 16'hf000;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`FLOUT1,`FLOUT1,`FLOUT1,`FLOUT1};
					state = M3;
				end
			end
			M3:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `FLOPTADR;
					o_wb_sel <= 16'h000f;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`FLOPT1,`FLOPT1,`FLOPT1,`FLOPT1};
					state = M4;
				end
			end
			M4:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= 0;
					o_wb_sel <= 0;
					o_wb_we  <= 0;
					o_wb_cyc  <= 0;
					o_wb_stb  <= 0;
					o_wb_dat <= 0;
				end
				
				if(i_irq)
				begin//Set all at once
					o_wb_adr <= `FLOPTADR;
					o_wb_sel <= 16'hffff;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= `FLALL;
					state = M5;
				end
				
			end
			M5:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= 0;
					o_wb_sel <= 0;
					o_wb_we  <= 0;
					o_wb_cyc  <= 0;
					o_wb_stb  <= 0;
					o_wb_dat <= 0;
				end
				if(i_irq)
					state = M6;
			end
			S1:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLINADR + 'h4;
					o_wb_sel <= 16'h00f0;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`SLIN2,`SLIN2,`SLIN2,`SLIN2};
					state = S2;
				end
			end
			S2:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLINADR + 'h8;
					o_wb_sel <= 16'h0f00;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`SLIN3,`SLIN3,`SLIN3,`SLIN3};
					state = S3;
				end
			end
			S3:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLINADR + 'hC;
					o_wb_sel <= 16'hf000;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`SLIN4,`SLIN4,`SLIN4,`SLIN4};
					state = S4;
				end
			end
			S4:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLOPTADR;
					o_wb_sel <= 16'hf000;
					o_wb_we  <= 1;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					o_wb_dat <= {`SLOPT,`SLOPT,`SLOPT,`SLOPT};
					state = S5;
				end
			end
			S5:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= 0;
					o_wb_sel <= 0;
					o_wb_we  <= 0;
					o_wb_cyc  <= 0;
					o_wb_stb  <= 0;
					o_wb_dat <= 0;
				end
				if(i_irq)
				begin//read all
					o_wb_adr <= `SLOUTADR;
					o_wb_sel <= 16'hffff;
					o_wb_we  <= 0;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					state = S6;
				end
			end
			S6:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLOUTADR;
					o_wb_sel <= 16'h000f;
					o_wb_we  <= 0;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					state = S7;
				end
			end
			S7:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLOUTADR + 'h4;
					o_wb_sel <= 16'h00f0;
					o_wb_we  <= 0;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					state = S8;
				end
			end
			S8:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLOUTADR + 'h8;
					o_wb_sel <= 16'h0f00;
					o_wb_we  <= 0;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					state = S9;
				end
			end
			S9:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= `SLOUTADR + 'hC;
					o_wb_sel <= 16'hf000;
					o_wb_we  <= 0;
					o_wb_cyc  <= 1;
					o_wb_stb  <= 1;
					state = Sa;
				end
			end
			Sa:
			begin
				if(i_wb_ack)
				begin
					o_wb_adr <= 0;
					o_wb_sel <= 0;
					o_wb_we  <= 0;
					o_wb_cyc  <= 0;
					o_wb_stb  <= 0;
					o_wb_dat <= 0;
				end
			end
		endcase
	end
end




endmodule

