//////////////////////////////////////////////////////////////////
//                                                              //
//  Slave Interface Module                                      //
//                                                              //
// Description                                                  //
//  This is a generated Accelerator interface module connected  //
//  To the Amber Cores wishbone interface in the SHMAC system   //
//                                                              //
//  Author(s):                                                  //
//      - Marton Teilgård, mteilgard@gmail.com		            //
//                                                              //
//////////////////////////////////////////////////////////////////


`include "common_defs.v"

//ACC IF parameters generated
//Script generated defines 
`define ACC_INPUTS		4
`define ACC_OUTPUTS		4

module IFM_slave_acc_dummy_4_4
   (
	
	output i_start,
output [31:0]i_opt,
input  o_rdy,
output [31:0]acc_in_0,
output [31:0]acc_in_1,
output [31:0]acc_in_2,
output [31:0]acc_in_3,

input [31:0]acc_out_0,
input [31:0]acc_out_1,
input [31:0]acc_out_2,
input [31:0]acc_out_3,
	
    input                  i_clk,
    input                  i_rst,
    input [31:0]           i_wb_adr,
    input [15:0]  		   i_wb_sel,
    input                  i_wb_we,
    output reg [127:0] 	   o_wb_dat,
    input [127:0]  		   i_wb_dat,
    input                  i_wb_cyc,
    input                  i_wb_stb,
    output                 o_wb_ack,
    output                 o_wb_err,
	output                 o_irq
	);
	
	
	reg [31:0] acc_in [0:(`ACC_INPUTS - 1)];
	reg [31:0] acc_out[0:(`ACC_OUTPUTS - 1)];
	reg [31:0] acc_opt;
	reg acc_start;
	wire acc_rdy;
	
	assign o_irq = acc_rdy; 
	//Output wires generated
	//wire [31:0] acc_out_0;
	//wire [31:0] acc_out_1;
	//wire [31:0] acc_out_2;
	//wire [31:0] acc_out_3;

	assign i_start = acc_start;
	assign i_opt = acc_opt;
	assign acc_rdy = o_rdy;
	assign acc_in_0 = acc_in[0];
	assign acc_in_1 = acc_in[1];
	assign acc_in_2 = acc_in[2];
	assign acc_in_3 = acc_in[3];
	
   wire                                         wb_start_write;
   wire                                         wb_start_read;
   reg                                          wb_start_read_d1 = 'd0;
   

   // ======================================================
   // Wishbone Interface
   // ======================================================

   // Can't start a write while a read is completing. The ack for the read cycle
   // needs to be sent first
   assign wb_start_write = i_wb_stb && i_wb_we && !wb_start_read_d1;
   assign wb_start_read  = i_wb_stb && !i_wb_we && !o_wb_ack;

   always @( posedge i_clk or posedge i_rst) begin
      if(i_rst)
        wb_start_read_d1 <= 1'b0;
      else
        wb_start_read_d1 <= wb_start_read;
   end

   assign o_wb_err = 1'd0;
   assign o_wb_ack = i_wb_stb && ( wb_start_write || wb_start_read_d1 );

   
	always @(posedge acc_rdy)
	begin
		acc_out[0] <= acc_out_0;
		acc_out[1] <= acc_out_1;
		acc_out[2] <= acc_out_2;
		acc_out[3] <= acc_out_3;
	end
   // ========================================================
   // Register Writes
   // ========================================================
	always @( posedge i_clk or posedge i_rst) begin
	if(i_rst) 
	begin//ACC IF generate this list
        acc_start <= 'h0;
	end
    else 
	begin
		if(acc_start)
			acc_start <= 0;
        if ( wb_start_write && !i_wb_adr[11])
			begin
				if(i_wb_sel[3])
					acc_in[{i_wb_adr[10:4],2'b00}] <= i_wb_dat[31:0];
				if(i_wb_sel[7])
					acc_in[{i_wb_adr[10:4], 2'b01}] <= i_wb_dat[63:32];
				if(i_wb_sel[11])
					acc_in[{i_wb_adr[10:4], 2'b10}] <= i_wb_dat[95:64];
				if(i_wb_sel[15])
				begin
					if(i_wb_adr[10:2] == 9'h1ff)
					begin
						acc_opt <= i_wb_dat[127:96];
						acc_start <= 1;
					end
					else
						acc_in[{i_wb_adr[10:4], 2'b11}] <= i_wb_dat[127:96];
				end
			end
		end
	end
   
	// ========================================================
   // Register Reads
   // ========================================================
   always @( posedge i_clk or posedge i_rst ) begin
      if(i_rst)
         o_wb_dat <= 127'h0;
      else 
		begin
			if ( wb_start_read && i_wb_adr[11])
			begin
				if(i_wb_sel[3])
					o_wb_dat[31:0] <= acc_out[{i_wb_adr[10:4],2'b00}];
				if(i_wb_sel[7])
					o_wb_dat[63:32] <= acc_out[{i_wb_adr[10:4],2'b01}];
				if(i_wb_sel[11])
					o_wb_dat[95:64] <= acc_out[{i_wb_adr[10:4],2'b10}];
				if(i_wb_sel[15])
					o_wb_dat[127:96] <= acc_out[{i_wb_adr[10:4],2'b11}];
			end
			else
				o_wb_dat[127:0] <= 128'b0;
	  end
   end

endmodule

