	(
		input          			i_clk,
		input          			i_rst,
		output reg 				o_irq,
		// Wishbone Master I/F
		output		[31:0]  	o_wbm_adr,
		output 		[15:0]	  	o_wbm_sel,
		output   		    	o_wbm_we,
		input 		[127:0]	  	i_wbm_dat,
		output 		[127:0]	 	o_wbm_dat,
		output      			o_wbm_cyc,
		output         			o_wbm_stb,
		input          			i_wbm_ack,
		input          			i_wbm_err,
		//wishbone Slave IF
		input 		[31:0]     	i_wbs_adr,
		input 		[15:0]  	i_wbs_sel,
		input               	i_wbs_we,
		output reg 	[127:0] 	o_wbs_dat,
		input 		[127:0] 	i_wbs_dat,
		input            	    i_wbs_cyc,
		input             	   	i_wbs_stb,
		output reg        	   	o_wbs_ack,
		output reg       	   	o_wbs_err
	);
	
	//statelist	
	localparam 	IDLE = 4'h0,
				REC = 4'h1,
				ADRS = 4'h2,
				FETCH = 4'h3,
				CALC = 4'h4,
				STORE = 4'h5,
				IRQ = 4'h6;
				
	//control registers
	reg [31:0]		options;
	reg [31:0]		main_i;
	reg [31:0]		in_adr;
	reg [31:0]		out_adr;
	reg [3:0]		state;
	reg 			reset_buffer;
	
	// accelerator control 
	reg acc_start;
	wire acc_rdy;
	wire [31:0] acc_opt;
	
	//wishbone master buffer IF
	reg [31:0] buf_input_adr;
	reg [31:0] buf_output_adr;
	reg buf_set_adr;
	reg buf_read_input;
	reg buf_write_output;
	reg buf_last_read;
	wire buf_stall;
	wire buf_rst;
	
	#0
		.i_clk(i_clk), 
		.i_rst(buf_rst), 
		.i_input_adr(in_adr), 
		.i_output_adr(out_adr), 
		.i_set_adr(buf_set_adr), 
		.i_read_input(buf_read_input), 
		.i_write_output(buf_write_output),
		.i_last_read(buf_last_read),
		.o_stall(buf_stall), 
		.o_wbm_adr(o_wbm_adr), 
		.o_wbm_sel(o_wbm_sel), 
		.o_wbm_we(o_wbm_we), 
		.i_wbm_dat(i_wbm_dat), 
		.o_wbm_dat(o_wbm_dat), 
		.o_wbm_cyc(o_wbm_cyc), 
		.o_wbm_stb(o_wbm_stb), 
		.i_wbm_ack(i_wbm_ack), 
		.i_wbm_err(i_wbm_err),
		#1
		.i_clk (i_clk),
		.i_rst (i_rst),
		.i_start (acc_start),
		.i_opt (options),
		.o_rdy (acc_rdy),
		#2
	);
	//assign signals
	assign buf_rst = i_rst || reset_buffer;
	
	//main operation
	always @(posedge i_clk)
	begin
		if(i_rst)
		begin
			o_wbs_dat <= 'h0;
			o_wbs_ack <= 'h0;
			o_wbs_err <= 'h0;
			options <= 'h0;
			main_i <= 'h0;
			in_adr <= 'h0;
			out_adr <= 'h0;
			state = IDLE;
			//wb buffer reset
			reset_buffer <= 0;
			buf_input_adr <= 'h0;
			buf_output_adr <= 'h0;
			buf_set_adr <= 'h0;
			buf_read_input <= 'h0;
			buf_write_output <= 'h0;
			buf_last_read <= 'h0;
			
		end//reset
		else
		begin
			case(state)
				IDLE:
				begin
					o_irq <= 0;
					reset_buffer <= 0;
					buf_last_read <= 0;
					if(i_wbs_stb)
						state = REC;
				end
				REC:
				begin
					if(i_wbs_stb)
					begin
						if(!o_wbs_ack)
						begin
							if(i_wbs_adr[11:4] == 8'h00)
							begin
								if(i_wbs_sel[3]) 
								begin
									options <= i_wbs_dat[31:0];
									buf_set_adr <= 1;
									state = ADRS; 
								end
								if(i_wbs_sel[7])
									main_i <= i_wbs_dat[63:32];
								if(i_wbs_sel[11])
									in_adr <= i_wbs_dat[95:64];
								if(i_wbs_sel[15])
									out_adr <= i_wbs_dat[127:96];
								o_wbs_ack <= 1;
							end
						end
						else
							o_wbs_ack <= 0;
					end
					else
						o_wbs_ack <= 0;						
				end//rec
				ADRS:
				begin
					buf_set_adr <= 0;
					o_wbs_ack <= 0;	
					if(!buf_stall)
					begin
						buf_read_input <= 1;
						if(main_i == 1)
							buf_last_read <= 1;
						state = FETCH;
					end
				end//ADRS
				FETCH:
				begin
					if(!buf_stall)
						if(buf_read_input)
							buf_read_input <= 0;
						else
						begin
							acc_start <= 1;
							state = CALC;
						end
				end//FETCH
				CALC:
				begin
					acc_start <= 0;
					if(acc_rdy)
					begin
						buf_write_output <= 1;
						main_i <= main_i -1;
						state = STORE;
					end
				end//CALC
				STORE:
				begin
					if(buf_write_output)
						buf_write_output <= 0;
					else
					begin
						if(main_i == 0)
							state = IRQ;
						else if(main_i == 1)
						begin
							if(!buf_stall)
							begin
								buf_read_input <= 1;
								buf_last_read <= 1;
								state = FETCH;
							end
						end
						else
						begin
							buf_read_input <= 1;	
							state = FETCH;
						end
					end
				end//STORE
				IRQ:
				begin
					if(!buf_stall)
					begin
						reset_buffer <= 1;
						o_irq <= 1;
						state = IDLE;
					end
				end //IRQ
			endcase
		end//normal op
	end//@i_clk
endmodule
