`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    16:52:09 05/03/2014 
// Design Name: 
// Module Name:    wb_arbiter 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: This is a basic two input, one output round robin wishbone master 
// arbiter. It checks on start and end of cycle on the wb_cyc signals of the two
// masters, and therefore it supports multi-read/write-cycle operations without interrupting
// the transfer.
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module wb_arbiter(
	input          					i_rst,
	
	// Wishbone Master I/F
	output		[31:0]  				o_wbm_adr,
	output 		[15:0]			  	o_wbm_sel,
	output   		     				o_wbm_we,
	input 		[127:0]			  	i_wbm_dat,
	output 		[127:0]			 	o_wbm_dat,
	output      		  				o_wbm_cyc,
	output         					o_wbm_stb,
	input          					i_wbm_ack,
	input          					i_wbm_err,
	
	//in0
	input		[31:0]  					in0_o_wbm_adr,
	input 		[15:0]			  	in0_o_wbm_sel,
	input   		     					in0_o_wbm_we,
	output 		[127:0]			  	in0_i_wbm_dat,
	input 		[127:0]			 	in0_o_wbm_dat,
	input      		  					in0_o_wbm_cyc,
	input         						in0_o_wbm_stb,
	output          					in0_i_wbm_ack,
	output          					in0_i_wbm_err,
	
	//in1
	input		[31:0]  					in1_o_wbm_adr,
	input 		[15:0]			  	in1_o_wbm_sel,
	input   		     					in1_o_wbm_we,
	output 		[127:0]			  	in1_i_wbm_dat,
	input 		[127:0]			 	in1_o_wbm_dat,
	input      		  					in1_o_wbm_cyc,
	input         						in1_o_wbm_stb,
	output          					in1_i_wbm_ack,
	output          					in1_i_wbm_err

);
	
	reg sel;
	reg cyc0;
	reg cyc1;
	wire cyc;
	
	//outputs
	assign o_wbm_adr = (sel) ? in1_o_wbm_adr : in0_o_wbm_adr;
	assign o_wbm_sel = (sel) ? in1_o_wbm_sel : in0_o_wbm_sel;
	assign o_wbm_we = (sel) ? in1_o_wbm_we : in0_o_wbm_we;
	assign o_wbm_dat = (sel) ? in1_o_wbm_dat : in0_o_wbm_dat;
	assign o_wbm_cyc = (sel) ? in1_o_wbm_cyc : in0_o_wbm_cyc;
	assign o_wbm_stb = (sel) ? in1_o_wbm_stb : in0_o_wbm_stb;

	//inputs
	assign in0_i_wbm_dat = (sel) ? 127'h0 : i_wbm_dat;
	assign in1_i_wbm_dat = (sel) ? i_wbm_dat: 127'h0;
	assign in0_i_wbm_ack = (sel) ? 0 : i_wbm_ack;
	assign in1_i_wbm_ack = (sel) ? i_wbm_ack: 0;
	assign in0_i_wbm_err = (sel) ? 0 : i_wbm_err;
	assign in1_i_wbm_err = (sel) ? i_wbm_err: 0;
	
	assign cyc = cyc0 || cyc1;
	
	always @*
	if(i_rst)
	begin
		sel <= 0;
		cyc0 <= 0;
		cyc1 <= 0;
	end
	else
	begin
		if (!cyc)
		begin
			if (in0_o_wbm_cyc)
			begin
				sel <= 0;
				cyc0 <= 1;
			end
			if (in1_o_wbm_cyc)
			begin
				sel <= 1;
				cyc1 <= 1;
			end
		end
		else
		begin
			if(cyc0)
			begin
				if (!in0_o_wbm_cyc)
					if (in1_o_wbm_cyc)
					begin
						sel <= 1;
						cyc1 <= 1;
					end
					else
						cyc0 <= 0;
			end
			if(cyc1)
			begin
				if (!in1_o_wbm_cyc)
					if (in0_o_wbm_cyc)
					begin
						sel <= 0;
						cyc0 <= 1;
					end
					else
						cyc1 <= 0;
			end
		end
	end
endmodule
